/******************************************************************************

    Filename:         decode.cpp

    Author:           Evan Weststrate

    Creation date:    28 January 2000

    Contents:         Main program that deciphers .dee files.


                          *** DOCUMENTATION ***

IMPROVEMENTS:
-add smt information 


 ******************************************************************************/

#include <fstream.h>
#include <string.h>
#include "codes.h"

#define NET_EXTENTION ".dee"
#define OUT_EXT ".extr"

//Function Declarations
bool PrintFile(char* filename);          //Prints the contents of a file; returns false if error
void SaveLine(ofstream* pFout,const char* type2,int FInum,const char* type1,
			  const char* pos,const int topNum);
					   //writes description line to opened file - fout
//Globals
const int MAX_LLGTH = 80;

int main(int argc, char *argv[])
{
  int i = 0;
  if (argc != 2)
    { 
      cout << "Usage: " << argv[0] << " <filename> " << endl;
      return 1;
    }

  char outFile[80];                                // set up I/O filenames
  char inFile[80];
  strncpy(outFile, argv[1],79);
  strncpy(inFile, argv[1],79);

  strcat(outFile, OUT_EXT);                        // output filename
  strcat(inFile, NET_EXTENTION);                   // input filename

  ifstream fin(inFile,ios::nocreate | ios::in);   // open the file specified by the 1st parameter
  if (fin.fail())
    {
      cout << "Could not open ";
      if (inFile)
	cout << inFile;
      cout << " for extraction.\n";
      return 1;
    }
 
  char verNum[10] = "0.0";
  fin.getline(verNum,MAX_LLGTH,'\n');                 //read version number
  if (strncmp(verNum,"3.0",3) != 0)                   //check ver. number
    {
      cout << verNum << " : Wrong version number for .dee file.\n";
      return 1;
    }

  char netName[80];
  fin >> netName;
  if (strncmp(netName,argv[1],strlen(argv[1])))       //check filename to netname
    {
      cout << "Network name does not match filename.\n";
      return 1;
    }

  int numOfDelays = 0;
  for (i=0;i<20;i++) {fin.ignore(MAX_LLGTH,'\n');}     //skip to delay information
  fin >> numOfDelays;

  int numOfElements = 0;
  for (i=0;i<12;i++) {fin.ignore(MAX_LLGTH,'\n');}     //skip to element information
  fin >> numOfElements;

  //  cout << "elements: " << numOfElements << endl;

  for (i=0;i<7;i++) {fin.ignore(MAX_LLGTH,'\n');}            //skip to topology
  for (i=0;i<numOfDelays;i++) {fin.ignore(MAX_LLGTH,'\n');}
  
  ofstream fout(outFile,ios::out);                    //open an output file (truncate it)
  if (fout.fail())
  {
	  cout << "Unable to open " << outFile << endl;
	  return 1;
  }
 
  int elPosition;
  int elType;
  int elType2;
  int junk;
  int FOnum;
  int FInum;
  ofstream *pFout = &fout;

  for (i=0;i<numOfElements;i++)
  {
	  fin >> elPosition >> elType;
	  fin.ignore(MAX_LLGTH,'\n');		      //next line
	  positioncode pCode(elPosition);
	  typecode tCode(elType);
	  switch (elType)
	  {
	  case 1:			              //wire
		  fin >> junk >> junk >> junk >> FOnum >> FInum;		//skip to FO#, FI#
 		  SaveLine(pFout," ",FInum,tCode.getName(),pCode.getName(),i);
		  break;
	  case 5:			              //gate
		  fin >> elType2;
		  fin >> junk >> junk >> junk >> FOnum >> FInum;		//skip to FO#, FI#
		  {gatecode gCode(elType2);
		  SaveLine(pFout,gCode.getName(),FInum,tCode.getName(),pCode.getName(),i);}
		  break;
	  case 15:			              //behavioral
		  fin >> elType2;
		  fin >> junk >> junk >> junk >> FOnum >> FInum;		//skip to FO#, FI#
		  {behavioralcode bCode(elType2);
		  SaveLine(pFout,bCode.getName(),FInum,tCode.getName(),pCode.getName(),i);}
		  break;

	  default:
		  fin >> junk;	                     //ignore type2 (have not dealt with yet)
		  fin >> junk >> junk >> junk >> FOnum >> FInum;		//skip to FO#, FI#
  		  SaveLine(pFout,"Unknown Type2",FInum,tCode.getName(),pCode.getName(),i);
	  }

	  fin.ignore(MAX_LLGTH,'\n');		      //next line
	  for (int j=0;j<(FOnum + FInum);j++) {
		  fin.ignore(MAX_LLGTH,'\n');}       //skip all links for each topo element
  }  //end topology parser

  fin.ignore(1000);

  fin.close();
  fout.close(); //close file after output-ing


//  if (PrintFile(inFile) == false) return 1;         //print input file
  if (PrintFile(outFile) == false) return 1;          //print outFile
//  cout << inFile << endl;  
  cout << "Listing saved to: " << outFile << endl;  

  return 0;
}

void SaveLine(ofstream* pFout,const char* type2,int FInum,const char* type1,
			  const char* pos,const int topNum)  
{
	pFout->width(6);
  	pFout->setf(ios::left,ios::adjustfield);
	*pFout << topNum;
	pFout->width(16);
	*pFout << pos;
	*pFout << type1;
	if (FInum != 0)
	  {
	    pFout->width(10); pFout->setf(ios::right,ios::adjustfield);
	    *pFout << FInum << " input ";
	  }
	pFout->width(13); pFout->setf(ios::left,ios::adjustfield);
	*pFout << type2 << endl;
	pFout->setf(ios::right);
}

bool PrintFile(char* filename) 
{
  ifstream fin(filename,ios::nocreate | ios::in);       // open the file
  if (fin.fail())  //is there error??
    {
      cout << "Could not open ";
	  cout << filename;
      cout << " for extraction.\n";
      return false;
    }
 
  cout << "***  "<< filename << " elements:  ***\n";
  char ch;
  while(fin.get(ch))  //display contents of file.
    cout << ch;
  cout << "\n";
  fin.close();  //to be safe.
  return true;
}
